#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Iterates over a bunch of .jpg or .cr2 files and matches 
DateTimeOriginal from Exif tag to DateTime in a csv log 
of a GeigerMuellerCounter."""

from datetime import datetime
import csv
import argparse
from PIL import Image
import piexif
import piexif.helper

# Constants for GQ geiger counters

# 300  series: 0.0065   µSv/h / CPM
# 320  series: 0.0065   µSv/h / CPM
# 500  series: 0.0065   µSv/h / CPM
# 500+ series: 0.0065   µSv/h / CPM for the first tube
# 600  series: 0.0065   µSv/h / CPM
# 600+ series: 0.002637 µSv/h / CPM

SIFACTOR = 0.0065 

parser = argparse.ArgumentParser(description='''A tool that writes 
    radiation levels (and optionally geocoordinates) to image files 
    and extracts the infos from external sources.''')
parser.add_argument('csv', metavar='CSV', type=str, 
                    help='Geiger counter history file in CSV format.')
parser.add_argument('photos', metavar='Photo', type=str, nargs='+', 
                    help='One or multiple photo image files to process.')

args = parser.parse_args()

for photo in args.photos:
    # Load Image and EXIF data
    im = Image.open(photo)
    exif_dict = piexif.load(im.info["exif"])

    picrawtime = exif_dict["Exif"][piexif.ExifIFD.DateTimeOriginal].decode('ASCII')
    picisotime = datetime.strptime(picrawtime, "%Y:%m:%d %H:%M:%S")

    print("Timestamp of image: ", picisotime)

    # Import GeigerCounter log
    with open("testdata/test.hisdb.his", "r") as f:
        csvreader = csv.reader(filter(lambda row: row[0]!='#', f), delimiter=',', skipinitialspace=True)
        for _, csvrawtime, csvrawcpm, _ in csvreader:
            csvisotime = datetime.fromisoformat(csvrawtime)
            
            if csvisotime == picisotime:
                rad = round(float(csvrawcpm) * SIFACTOR, 2)
                print("Radiation at time of photo: ", rad, "µS/h")

                # convert str to exif compatible string
                new_comment = "Radiation ☢ " + str(rad) + " µS/h"
                user_comment = piexif.helper.UserComment.dump(new_comment, encoding="unicode")
                exif_dict["Exif"][piexif.ExifIFD.UserComment] = user_comment
                
                # compile and write tags
                print("Going to write comment: " + new_comment)
                exif_bytes = piexif.dump(exif_dict)
                im.save("testdata/test.jpg", exif=exif_bytes)
                break
